# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.pysped.xml_sped import TagCaracter, TagInteiro, TagDecimal, TagDataHoraUTC, Signature
from nfe.mdfe.manual_300.evIncCondutorMDFe import EvIncCondutorMDFe
from nfe.mdfe.manual_300.evCancMDFe import EvCancMDFe
from nfe.mdfe.manual_300.evEncMDFe import EvEncMDFe
from nfe.mdfe.base import XMLMDFe, NAMESPACE_MDFE
from nfe.mdfe.manual_300 import *
import os

DIRNAME = os.path.dirname(__file__)


class DetEvento(XMLMDFe):
    """Informacoes do evento específico."""
    def __init__(self):
        super(DetEvento, self).__init__()
        self.evCancelamento = EvCancMDFe()
        self.evEncerramento = EvEncMDFe()
        self.evIncCondutorMDFe = EvIncCondutorMDFe()

    @property
    def xml(self):
        xml = u'<detEvento>'
        xml += self.evCancelamento.xml
        xml += self.evEncerramento.xml
        xml += self.evIncCondutorMDFe.xml
        xml += u'</detEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.evCancelamento.xml = arquivo
        self.evEncerramento.xml = arquivo
        self.evIncCondutorMDFe.xml = arquivo


class InfEvento(XMLMDFe):
    """Grupo de informacoes do registro do evento."""
    def __init__(self):
        super(InfEvento, self).__init__()
        self.Id = TagCaracter(
            nome=u'infEvento', tamnho=[48, 48], propriedade=u'Id', raiz=u'//infEvento')
        self.cOrgao = TagInteiro(nome=u'cOrgao', tamanho=[2, 2], raiz=u'//infEvento')
        self.tpAmb = TagInteiro(nome=u'tpAmb', tamanho=[1, 1], raiz=u'//infEvento')
        self.cnpj = TagInteiro(nome=u'CNPJ', tamanho=[14, 14], raiz=u'//infEvento')
        self.chMDFe = TagInteiro(nome=u'chMDFe', tamanho=[44, 44], raiz=u'//infEvento')
        self.dhEvento = TagDataHoraUTC(nome=u'dhEvento', raiz=u'//infEvento')
        self.tpEvento = TagInteiro(nome=u'tpEvento', tamanho=[6, 6], raiz=u'//infEvento')
        self.nSeqEvento = TagInteiro(nome=u'nSeqEvento', tamanho=[1, 2], raiz=u'//infEvento')
        self.detEvento = DetEvento()

    @property
    def xml(self):
        xml = u'<infEvento Id="%s">' % self.Id.valor
        xml += self.cOrgao.xml
        xml += self.tpAmb.xml
        xml += self.cnpj.xml
        xml += self.chMDFe.xml
        xml += self.dhEvento.xml
        xml += self.tpEvento.xml
        xml += self.nSeqEvento.xml
        xml += self.detEvento.xml
        xml += u'</infEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.Id.xml = arquivo
        self.cOrgao.xml = arquivo
        self.tpAmb.xml = arquivo
        self.cnpj.xml = arquivo
        self.chMDFe.xml = arquivo
        self.dhEvento.xml = arquivo
        self.tpEvento.xml = arquivo
        self.nSeqEvento.xml = arquivo
        self.detEvento.xml = arquivo


class EventoMDFe(XMLMDFe):
    """Mensagem de solicitacao de registro de evento do MDF-e."""
    def __init__(self):
        super(EventoMDFe, self).__init__()
        self.versao = TagDecimal(
            nome=u'eventoMDFe', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade=u'versao',
            raiz=u'/', namespace=NAMESPACE_MDFE, valor=u'3.00')
        self.infEvento = InfEvento()
        self.signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'eventoMDFe_v3.00.xsd'

    @property
    def xml(self):
        xml = u'<eventoMDFe xmlns="%s" versao="%s">' % (NAMESPACE_MDFE, self.versao.valor)
        xml += self.infEvento.xml
        xml += self.signature.xml
        xml += u'</eventoMDFe>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.infEvento.xml = arquivo
        self.signature.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
