# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.mdfe.manual_300.retConsStatServ import RetConsStatServMDFe
from nfe.mdfe.manual_300.retConsReciMDFe import RetConsReciMDFe
from nfe.mdfe.manual_300.consStatServ import ConsStatServMDFe
from nfe.mdfe.manual_300.soap import SOAPEnvio, SOAPRetorno
from nfe.mdfe.manual_300.consReciMDFe import ConsReciMDFe
from nfe.mdfe.manual_300.retEnviMDFe import RetEnviMDFe
from nfe.pysped.xml_sped.certificado import Certificado
from nfe.mdfe.manual_300.enviMDFe import EnviMDFe
from nfe.mdfe.manual_300.webservices import *
from unicodedata import normalize, category
from nfe.pysped.xml_sped import ABERTURA
from datetime import datetime
import requests
import tempfile
import logging
import sys
import os
if sys.version_info.major == 3:
    unicode = str

PEM = os.path.dirname(
    os.path.abspath(__file__)) + '/../../pysped/xml_sped/cadeia-certificadora/cadeia.pem'


class CertificadoMDFe(Certificado):
    def __init__(self):
        super(CertificadoMDFe, self).__init__()

    def _prepara_doc_xml(self, xml):
        if isinstance(xml, str):
            xml = unicode(xml.encode('utf-8'))

        # Determina o tipo de arquivo que vai ser assinado, procurando pela tag correspondente
        if u'infMDFe' in xml:
            doctype = u'<!DOCTYPE MDFe [<!ATTLIST infMDFe Id ID #IMPLIED>]>'
        elif u'infEvento' in xml:
            doctype = u'<!DOCTYPE envEvento [<!ATTLIST infEvento Id ID #IMPLIED>]>'
        else:
            raise ValueError('Tipo de arquivo desconhecido para assinatura/validacao')

        xml = xml.replace(ABERTURA, ABERTURA + doctype)
        xml = xml.replace(u'\n', u'')

        return xml

    def _finaliza_xml(self, xml):
        # Determina o tipo de arquivo que vai ser assinado, procurando pela tag correspondente
        if u'infMDFe' in xml:
            doctype = u'<!DOCTYPE MDFe [<!ATTLIST infMDFe Id ID #IMPLIED>]>'
        elif u'infEvento' in xml:
            doctype = u'<!DOCTYPE envEvento [<!ATTLIST infEvento Id ID #IMPLIED>]>'
        else:
            raise ValueError('Tipo de arquivo desconhecido para assinatura/validacao')

        xml = xml.replace(ABERTURA + doctype, ABERTURA)
        xml = xml.replace(u'\n', u'')

        return xml


class ProcessoMDFe(object):
    def __init__(self, servico, envio, resposta):
        self.servico = servico
        self.envio = envio
        self.resposta = resposta

    def __str__(self):
        if self.resposta.cStat.valor:
            motivo = self.resposta.xMotivo.valor
            motivo = unicode(motivo) if type(motivo) == str else motivo
            motivo = unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', motivo)))
            return '[%s] %s' % (self.resposta.cStat.valor, motivo)

        return ''

    def __unicode__(self):
        if self.resposta.cStat.valor:
            return u'[%s] %s' % (self.resposta.cStat.valor, self.resposta.xMotivo.valor)

        return u''


class ProcessadorMDFe(object):
    logger = logging.getLogger('MDFe')  # Registro de log da classe

    def __init__(self, cert, chave, ambiente, estado):
        """
        Metodo construtor da classe
        :param cert: certificado digital da empresa
        :param chave: chave privada para o uso do certificado
        :param ambiente: ambiente de atuacao da MDFe: 1=producao ou 2=homologacao
        :param estado: codigo IBGE do estado (UF)
        """
        self._cert = cert
        self._chave = chave
        self._ambiente = ambiente
        self._estado = estado
        self.tentativas = 5

    def conectar(self, servico, envio, resposta):
        """Conexao com o webservice"""
        # Tratamento do certificado digital e da chave privada
        cert, chave = tempfile.NamedTemporaryFile('w'), tempfile.NamedTemporaryFile('w')
        cert.write(self._cert)
        cert.flush()
        chave.write(self._chave)
        chave.flush()
        _cert = cert.name, chave.name

        # Le parametros do webservice
        ws = WEBSERVICE[self._estado][self._ambiente][servico]

        # Dados de envio
        body = SOAPEnvio()
        body.servico = ws['servico']
        body.cUF = self._estado
        body.dados = envio
        body = body.xml.encode('utf-8')
        content_type = u'application/soap+xml; charset=utf-8; ' \
                       u'action="http://www.portalfiscal.inf.br/mdfe/wsdl/%s"'
        header = {'content-type': content_type % ws['servico']}

        # Dados de retorno
        soap_retorno = SOAPRetorno()
        soap_retorno.servico = ws['servico']
        soap_retorno.metodo = ws['metodo']
        soap_retorno.resposta = resposta

        # Conectar servico
        for t in range(1, self.tentativas + 1):
            try:
                result = requests.post(
                    url=ws['url'], data=body, cert=_cert, verify=PEM, headers=header)

                if result.ok:
                    soap_retorno.resposta.xml = result.content.decode('utf-8')
                    break
                else:
                    resposta.alertas.append((t, '%s-%s' % (result.status_code, result.reason)))
            except Exception as err:
                resposta.alertas.append((t, err.__str__()))

        cert.close()
        chave.close()

    def consultar_servico(self):
        """Pedido de consulta do status do Servico MDF-e."""
        envio = ConsStatServMDFe()
        resposta = RetConsStatServMDFe()
        envio.tpAmb.valor = self._ambiente
        envio.validar()  # Try, if error append process alerts
        self.logger.info(u'*** consultar servico ***')
        processo = ProcessoMDFe(MDFE_STATUS_SERVICO, envio, resposta)
        self.conectar(MDFE_STATUS_SERVICO, envio, resposta)

        return processo

    def enviar_lote(self, lote):
        """Envio de lote de MDF-e."""
        envio = EnviMDFe()
        resposta = RetEnviMDFe()
        cert_mdfe = CertificadoMDFe()
        cert_mdfe.chave = self._chave
        cert_mdfe.certificado = self._cert
        lote = lote if isinstance(lote, (list, tuple)) else [lote]
        for mdfe in lote:
            mdfe.signature.xml = cert_mdfe.assina_xml(mdfe.xml)
            mdfe.validar()
        envio.mdfe = lote
        envio.idLote.valor = "{0}{1}".format(
            unicode(self._ambiente), datetime.now().strftime('%Y%m%d%H%M%S'))
        envio.validar()
        processo = ProcessoMDFe(MDFE_STATUS_SERVICO, envio, resposta)
        self.logger.info(u'*** enviar lote ***')
        self.conectar(MDFE_RECEPCAO, envio, resposta)

        return processo

    def consultar_recibo(self, recibo):
        """Devolve o resultado do processamento do MDF-e. (envio de lote)"""
        envio = ConsReciMDFe()
        resposta = RetConsReciMDFe()
        envio.nRec.valor = recibo
        envio.tpAmb.valor = self._ambiente
        envio.validar()
        processo = ProcessoMDFe(MDFE_RET_RECEPCAO, envio, resposta)
        self.logger.info(u'*** consultar recibo ***')
        self.conectar(MDFE_RET_RECEPCAO, envio, resposta)
        if processo.resposta.cStat.valor == 104:  # Arquivo processado
            processo.resposta.cStat.valor = processo.resposta.protMDFe.infProt.cStat.valor
            processo.resposta.xMotivo.valor = processo.resposta.protMDFe.infProt.xMotivo.valor

        return processo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
