# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
import os
from nfe.pysped.xml_sped import *
from nfe.pysped.nfe.manual_500 import conssitnfe_310
from nfe.nfe_400.manual_600.nfe_400 import ESQUEMA_ATUAL
from nfe.nfe_400.manual_600.consrecinfe import ProtNFe
from nfe.nfe_400.manual_600.cancnfe import RetCancNFe

DIRNAME = os.path.dirname(__file__)


class ConsSitNFe(conssitnfe_310.ConsSitNFe):
    def __init__(self):
        super(ConsSitNFe, self).__init__()
        self.versao = TagDecimal(nome=u'consSitNFe', codigo=u'EP01', propriedade=u'versao',
                                 namespace=NAMESPACE_NFE, valor=u'4.00', raiz=u'/')
        self.caminho_esquema = os.path.join(DIRNAME, u'schema', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'consSitNFe_v4.00.xsd'


class InfEvento(XMLNFe):
    def __init__(self):
        super(InfEvento, self).__init__()
        self.cOrgao = TagCaracter(nome=u'cOrgao', codigo=u'', tamanho=[2, 2, 2])
class Evento(XMLNFe):
    def __init__(self):
        super(Evento, self).__init__()
        self.infEvento = InfEvento
        self.signature = Signature()

class ProcEventoNFe(XMLNFe):
    def __init__(self):
        super(ProcEventoNFe, self).__init__()
        self.evento = Evento()
        self.retEvento = RetEvento()

    @property
    def xml(self):
        xml = u'<procEventoNFe>'
        xml += self.evento.xml
        xml += self.retEvento.xml

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.evento.xml = arquivo
        self.retEvento.xml = arquivo


class RetConsSitNFe(conssitnfe_310.RetConsSitNFe):
    def __init__(self):
        super(RetConsSitNFe, self).__init__()
        self.versao = TagDecimal(nome=u'retConsSitNFe', codigo=u'ER01', propriedade=u'versao',
                                 namespace=NAMESPACE_NFE, valor=u'4.00', raiz=u'/')
        self.dhRecbto = TagDataHoraUTC(nome=u'dhRecbto', codigo=u'ER07a', raiz=u'/')
        self.procEventoNFe = []
        self.caminho_esquema = os.path.join(DIRNAME, u'schema', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'retConsSitNFe_v4.00.xsd'

    @property
    def xml(self):
        xml = ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml
        xml += self.chNFe.xml
        if self.protNFe is not None:
            xml += self.protNFe.xml
        if self.retCancNFe is not None:
            xml += tira_abertura(self.retCancNFe.xml)
        for p in self.procEventoNFe:
            xml += p.xml
        xml += u'</retConsSitNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.cUF.xml = arquivo
            self.dhRecbto.xml = arquivo
            self.chNFe.xml = arquivo
            if self._le_noh(u'//retConsSitNFe/protNFe'):
                self.protNFe = ProtNFe()
                self.protNFe.xml = arquivo
            if self._le_noh(u'//retConsSitNFe/retCancNFe'):
                self.retCancNFe = RetCancNFe()
                self.retCancNFe.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
